{ TagFactory - Tagline manipulation program - by Jon Hunt  }
{   (c)1998-1999 Concilium Orb                             }
{ For external procedures and functions, see Procs.pas     }
unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, StdCtrls, ExtCtrls, Buttons, IniFiles, Consts;

type
  TMain_Frm = class(TForm)
    Bevel: TBevel;
    Scr: TMemo;
    Status: TStatusBar;
    OpenDialog: TOpenDialog;
    Merge_Picker: TOpenDialog;
    Exit_Btn: TBitBtn;
    Open_Btn: TBitBtn;
    Srch_Btn: TBitBtn;
    Sort_Btn: TBitBtn;
    Dupe_Btn: TBitBtn;
    Clr_Btn: TBitBtn;
    Abt_Btn: TBitBtn;
    Dump_Btn: TBitBtn;
    Edit_Btn: TBitBtn;
    Conf_Btn: TBitBtn;
    Merge_Btn: TBitBtn;
    Repl_Btn: TBitBtn;
    Long_Btn: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Open_BtnClick(Sender: TObject);
    procedure OpenDialogSelectionChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Exit_BtnClick(Sender: TObject);
    procedure Srch_BtnClick(Sender: TObject);
    procedure Sort_BtnClick(Sender: TObject);
    procedure Dupe_BtnClick(Sender: TObject);
    procedure Clr_BtnClick(Sender: TObject);
    procedure Abt_BtnClick(Sender: TObject);
    procedure Conf_BtnClick(Sender: TObject);
    procedure Dump_BtnClick(Sender: TObject);
    procedure Edit_BtnClick(Sender: TObject);
    procedure Merge_BtnClick(Sender: TObject);
    procedure Repl_BtnClick(Sender: TObject);
    procedure Long_BtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

const
  VERNO = 'v3.0';

var
  Main_Frm   : TMain_Frm;
  IniFile    : TIniFile;           // Inifile
  lSearchOk  : Boolean;            // Used for stopping search
  lDefFile   : Boolean;
  lDefDump   : Boolean;            // Has a default file been specified
  cDefFile   : string;
  cDefDump   : string;
  cBakFile   : string;             // Default files from Ini
  cEditor    : string;             // Editor to use
  cCurrDir   : string;             // Where we are running program
  lSplit     : Boolean;            // Split dump by deafult?
  lAvery     : Boolean;            // Do we include Avery markers
  liSearch   : TStrings;           // Search strings for S&R
  liReplace  : TStrings;           // Replace strings for S&R
  iLines     : integer;            // No of lines to split at
  iTotalTags : integer;            // Total taglines in current tagfile
  cKeyWord   : string;             // The word we search for in tag files
  cParam     : string;             // What parameter was used
  lParam     : Boolean;            // Did we run with a parameter
  iMaxLen    : integer;            // Tags longer than this, get trashed
  iDo        : integer;            // What to do with Parameters

implementation

uses About, Config, Procs, Search, Dump;

{$R *.DFM}

{ Set things up when we startup program }
procedure TMain_Frm.FormCreate(Sender: TObject);
var
  iLength: integer;                // Length of directory name
begin
  // Check for parameters
  iDo := 0;
  if ParamCount <> 0 then
    begin
      cParam := ParamStr(1);
      lParam := True;
    end
  else lParam := False;
  Main_Frm.Caption := 'TagFactory '+VERNO;
  cCurrDir := ExtractFilePath(ParamStr(0));
  // Did we run from the root directory?
  if Pos('\',cCurrDir) = Length(cCurrDir) then
    begin
      iLength := Length(cCurrDir);
      Delete(cCurrDir,iLength,1);
    end;
  IniFile := TIniFile.Create(AddName2Path(cCurrDir,'TFACTORY.INI'));
end;


{ Read Inifile and show current settings }
procedure TMain_Frm.FormShow(Sender: TObject);
var
  cSplit     : string;               // Split file?
  cAvery     : string;               // Avery markers?
  iBakFile   : integer;              // File handle for backup file
  liTempSrch : TStrings;             // Temp list of Srch strings
  liTempRepl : TStrings;             // Temp list of replace strings
  n          : integer;              // Counter for String Lists
begin
  cDefFile := IniFile.ReadString('Defaults','TagFile','');
  cDefDump := IniFile.ReadString('Defaults','DumpFile','');
  cBakFile := IniFile.ReadString('Backup','BackupFile','');
  cEditor  := IniFile.ReadString('Preferences','Editor','');
  cSplit   := IniFile.ReadString('Preferences','Split','');
  cAvery   := IniFile.ReadString('Preferences','AveryMarkers','');
  iLines   := IniFile.ReadInteger('Preferences','SplitLines',100);
  iMaxLen  := IniFile.ReadInteger('Preferences','MaxLength',500);
  iTotalTags := IniFile.ReadInteger('Totals','TagCount',0);

  // make sure a backup file is specified
  if cBakFile = '' then
    begin
      MessageDlg('No Backup File Specified In TFACTORY.INI.  Unable To Continue!',
           mtError,[mbOK],0);
      Close;
    end;

  // and make sure it actually exists, if not we create it
  if FileExists(cBakFile) = False then
    begin
      MessageDlg('Backup File Specified In TFACTORY.INI Does Not Exist!  Creating...',
           mtWarning,[mbOK],0);
      iBakFile := FileCreate(cBakFile);
      FileClose(iBakFile);
    end;

  // Setup the search and replace lists
  liSearch   := TStringList.Create;
  liReplace  := TStringList.Create;
  liTempSrch := TStringList.Create;
  liTempRepl := TStringList.Create;
  IniFile.ReadSectionValues('Search',liTempSrch);
  IniFile.ReadSectionValues('Replace',liTempRepl);

  // Now we extract the strings from the keys
  n := 1;
  while n <= liTempSrch.Count do
    begin
      liSearch.Add(liTempSrch.Values['Search'+IntToStr(n)]);
      Inc(n);
    end;
  n := 1;
  while n <= liTempRepl.Count do
    begin
      liReplace.Add(liTempRepl.Values['Replace'+IntToStr(n)]);
      Inc(n);
    end;

  if cDefFile = '' then lDefFile := False
  else lDefFile := True;

  if lDefFile = True then
    begin
      Status.Panels[0].Text := UpperCase(cDefFile);
      // Warn if no such tagline file
      if FileExists(cDefFile) = False then
        begin
          MessageDlg('Tagline File Specified In TFACTORY.INI Can Not Be Found!',
               mtWarning,[mbOK],0);
        end
      else
        begin
          Status.Panels[1].Text := IntToStr(iTotalTags);;
        end;
    end;
  liTempSrch.Free;
  liTempRepl.Free;

  if cDefDump = '' then lDefDump := False
  else lDefDump := True;

  if cSplit = 'False' then lSplit := False
  else lSplit := True;

  if cAvery = 'False' then lAvery := False
  else lAvery := True;

  // Use notepad if nothing else specified
  if cEditor = '' then cEditor := 'notepad.exe';

  // If we ran with a parameter, then work out which one
  if lParam = True then WhichParam;

  case iDo of
    1: SortTags;
    2: DeDupeTags;
    3: SearchReplace;
    4: RemoveLongOnes;
  end;

  // if we ran with a valid parameter, quit when done.
  if lParam = True then
    begin
      Main_Frm.WindowState := wsMinimized;
      Application.Terminate;
    end;
end;


{ Quitting TagFactory }
procedure TMain_Frm.Exit_BtnClick(Sender: TObject);
begin
  if MessageDlg('Are You Sure You Want To Quit TagFactory?',mtConfirmation,
         [mbYes,mbNo],0) = mrYes then
    begin
      Close;
    end;
end;


{ Destroy form and exit cleanly }
procedure TMain_Frm.FormDestroy(Sender: TObject);
begin
  CloseProg;
end;


{ Configuration of TagFactory }
procedure TMain_Frm.Conf_BtnClick(Sender: TObject);
begin
  Config_Frm.Show;
  Main_Frm.Hide;
end;


{ Open Tagline File }
procedure TMain_Frm.Open_BtnClick(Sender: TObject);
begin
  OpenDialog.Execute;
  lDefFile := False;
end;


{ Select multiple files and merge with main file }
procedure TMain_Frm.Merge_BtnClick(Sender: TObject);
var
  fMergeFile : TextFile;            // File we are merging to
  fCurrFile  : TextFile;            // The file we are working on at present
  cMergeFile : string;              // Merge file in string format
  iFiles     : integer;             // Counter for files we are merging from
  cLines     : string;              // Lines we are merging
begin
  if (OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected To Merge To!',mtError,[mbOK],0);
      Exit;
    end;

  Merge_Picker.Execute;

  // Just quit if no files were picked
  if Merge_Picker.Files.Count = 0 then
    begin
      Exit;
    end;

  Status.Panels[2].Text := 'Merging Files...';
  Screen.Cursor := crHourGlass;

  if lDefFile = True then
    begin
      AssignFile(fMergeFile,cDefFile);
      cMergeFile := cDefFile;
    end
  else
    begin
      AssignFile(fMergeFile,OpenDialog.Filename);
      cMergeFile := OpenDialog.FileName;
    end;

  // Backup tagline file
  CopyFile(cMergeFile,cBakFile);

  iFiles := 0;

  repeat
    AssignFile(fCurrFile,Merge_Picker.Files.Strings[0]);
    Application.ProcessMessages;
    Append(fMergeFile);
    Reset(fCurrFile);
    while not Eof(fCurrFile) do
      begin
        Readln(fCurrFile,cLines);
        Writeln(fMergeFile,cLines);
      end;
    CloseFile(fCurrFile);
    CloseFile(fMergeFile);
    Inc(iFiles);
  until iFiles = Merge_Picker.Files.Count;

  Status.Panels[2].Text := 'Idle';
  Screen.Cursor := crDefault;
  MessageDlg(IntToStr(Merge_Picker.Files.Count)+
       ' File(s) Have Been Merged Into Your Defautlt TagFile',mtInformation,
       [mbOK],0);
  CountTags(cMergeFile);
end;


{ Change caption of status bar on new file }
procedure TMain_Frm.OpenDialogSelectionChange(Sender: TObject);
begin
  Status.Panels[0].Text := UpperCase(OpenDialog.FileName);
  CountTags(OpenDialog.FileName);
end;


{ Search for taglines }
procedure TMain_Frm.Srch_BtnClick(Sender: TObject);
begin
  if (OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;
  Main_Frm.Enabled := False;
  Srch_Frm.Show;
end;


{ Sort tagline file }
procedure TMain_Frm.Sort_BtnClick(Sender: TObject);
begin
  SortTags;
end;


{ De-Dupe Tagline file }
procedure TMain_Frm.Dupe_BtnClick(Sender: TObject);
begin
  DeDupeTags;
end;


{ Clear tagline window }
procedure TMain_Frm.Clr_BtnClick(Sender: TObject);
begin
  Scr.Lines.Clear;
end;


{ Edit tagline file }
procedure TMain_Frm.Edit_BtnClick(Sender: TObject);
var
  cFile : string;
begin
  if (OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;

  if lDefFile = True then cFile := cDefFile
  else cFile := Main_Frm.OpenDialog.Filename;

  ExecuteFile(cEditor,cFile,'',SW_SHOWMAXIMIZED);
  CountTags(cFile);
end;


{ Dump Taglines To File }
procedure TMain_Frm.Dump_BtnClick(Sender: TObject);
begin
  if Scr.Lines.Count = 0 then
    begin
      MessageDlg('No Taglines To Export!',mtError,[mbOK],0);
      Exit;
    end;
  Main_Frm.Enabled := False;
  Dump_Frm.Show;
end;


{ Search & Replace }
procedure TMain_Frm.Repl_BtnClick(Sender: TObject);
begin
  SearchReplace;
end;


{ About Box }
procedure TMain_Frm.Abt_BtnClick(Sender: TObject);
begin
  Main_Frm.Enabled := False;
  About_Frm.Show;
end;


{ Remove Long taglines }
procedure TMain_Frm.Long_BtnClick(Sender: TObject);
begin
  RemoveLongOnes;
end;


end.
