{ External functions and procedures for TagFactory }

unit Procs;

interface

uses
  Windows, SysUtils, Classes, Controls, Forms, Dialogs, Consts, ShellAPI;

function ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd:
   integer): THandle;
function HasAttr(const FileName: string; Attr: Word): Boolean;
function AddName2Path(cPath, cName :string) :string;
function StrpPunc(cStrip: string): string;
function ChangeExt(FileName, NewExt: string): string;
function StrRight(cText: string; nChars: integer): string;
function BackupTags(cTagFileName: string): Boolean;
function CountTags(TagFile: string): integer;
procedure CopyFile(const FileName, DestName: TFileName);
procedure CloseProg;
procedure WhichParam;
procedure SortTags;
procedure DeDupeTags;
procedure SearchReplace;
procedure RemoveLongOnes;

implementation

uses Main, Sort, Dupe, Replace, TooLong;

{ Runs an external program }
function ExecuteFile(const FileName, Params, DefaultDir: string; ShowCmd:
   integer): THandle;
var
  zFileName, zParams, zDir: array[0..79] of Char;
begin
  Result := ShellExecute(Application.MainForm.Handle,nil,StrPCopy(zFileName,
        FileName),StrPCopy(zParams,Params),StrPCopy(zDir,DefaultDir),
        ShowCmd);
end;


{ Copy a file }
procedure CopyFile(const FileName, DestName: TFileName);
var
  CopyBuffer   : Pointer;    // buffer for copying }
  BytesCopied  : Longint;
  Source, Dest : Integer;    // handles
  Destination  : TFileName;  // holder for expanded destination name
const
  ChunkSize    : Longint = 8192; // copy in 8K chunks
begin
  Destination := ExpandFileName(DestName);  // expand the destination path
  if HasAttr(Destination,faDirectory) then // if destination is a directory...
    Destination := Destination + '\' + ExtractFileName(FileName); // ...clone file name
  GetMem(CopyBuffer,ChunkSize);  // allocate the buffer

  try
    Source := FileOpen(FileName,fmShareDenyWrite);   // open source file
    if Source < 0 then raise EFOpenError.CreateFmt(SFOpenError,[FileName]);
    try
      Dest := FileCreate(Destination);   // create output file; overwrite existing
      if Dest < 0 then raise EFCreateError.CreateFmt(SFCreateError,[Destination]);
      try
        repeat
          BytesCopied := FileRead(Source,CopyBuffer^,ChunkSize);  // read chunk
          if BytesCopied > 0 then   // if we read anything...
            FileWrite(Dest,CopyBuffer^,BytesCopied);  // ...write chunk
        until BytesCopied < ChunkSize;  // until we run out of chunks
      finally
        FileClose(Dest);    // close the destination file
      end;
    finally
      FileClose(Source);     // close the source file
    end;
  finally
    FreeMem(CopyBuffer,ChunkSize);   // free the buffer
  end;
end;


{ Get a files attributes }
function HasAttr(const FileName: string; Attr: Word): Boolean;
begin
  Result := (FileGetAttr(FileName) and Attr) = Attr;
end;


{ Add filenames to paths }
function AddName2Path(cPath, cName :string): string;
begin
  Result := cPath;
  if Result[Length(Result)] <> '\' then Result := Result + '\';
  Result := Result + cName;
end;


{ Removes filename extension & replaces it }
function ChangeExt(FileName, NewExt: string): string;
var
  n : integer;
begin
  n := Pos('.',FileName);
  n := n + 1;
  Delete(FileName, n, 4);
  Result := Concat(FileName,NewExt);
end;


{ takes the rightmost n characters from a string of any length.
  n is passed in }
function StrRight(cText: string; nChars: integer): string;
var
  nLen, nStart: integer;
begin
  nLen := Length(cText);
  nstart := nLen - nChars + 1;
  Result := Copy(cText,nStart,nChars);
end;


{ Free memory and shutdown }
procedure CloseProg;
begin
  IniFile.Free;
  liSearch.Free;
  liReplace.Free;
  Application.Terminate;
end;


{ Which Parameter was used }
procedure WhichParam;
begin
  if (UpperCase(cParam) = '/S') or (UpperCase(cParam) = '/D') or
     (UpperCase(cParam) = '/R') or (UpperCase(cParam) = '/L') then
    begin
      if UpperCase(cParam) = '/S' then iDo := 1;
      if UpperCase(cParam) = '/D' then iDo := 2;
      if UpperCase(cParam) = '/R' then iDo := 3;
      if UpperCase(cParam) = '/L' then iDo := 4;
    end

  else lParam := False;
end;


{ Strips all punctuation etc from a string }
function StrpPunc(cStrip: string): string;
begin
  while Pos(' ',cStrip) > 0 do delete(cStrip,Pos(' ',cStrip),1);
  while Pos('.',cStrip) > 0 do delete(cStrip,Pos('.',cStrip),1);
  while Pos(',',cStrip) > 0 do delete(cStrip,Pos(',',cStrip),1);
  while Pos('!',cStrip) > 0 do delete(cStrip,Pos('!',cStrip),1);
  while Pos('?',cStrip) > 0 do delete(cStrip,Pos('?',cStrip),1);
  while Pos('"',cStrip) > 0 do delete(cStrip,Pos('"',cStrip),1);
  while Pos('-',cStrip) > 0 do delete(cStrip,Pos('-',cStrip),1);
  while Pos('_',cStrip) > 0 do delete(cStrip,Pos('_',cStrip),1);
  while Pos('@',cStrip) > 0 do delete(cStrip,Pos('@',cStrip),1);
  while Pos('#39',cStrip) > 0 do delete(cStrip,Pos('#39',cStrip),1);
  while Pos('/',cStrip) > 0 do delete(cStrip,Pos('/',cStrip),1);
  while Pos('#',cStrip) > 0 do delete(cStrip,Pos('#',cStrip),1);
  while Pos('(',cStrip) > 0 do delete(cStrip,Pos('(',cStrip),1);
  while Pos(')',cStrip) > 0 do delete(cStrip,Pos(')',cStrip),1);
  while Pos('^',cStrip) > 0 do delete(cStrip,Pos('^',cStrip),1);
  while Pos('<',cStrip) > 0 do delete(cStrip,Pos('<',cStrip),1);
  while Pos('>',cStrip) > 0 do delete(cStrip,Pos('>',cStrip),1);
  while Pos('[',cStrip) > 0 do delete(cStrip,Pos('[',cStrip),1);
  while Pos(']',cStrip) > 0 do delete(cStrip,Pos(']',cStrip),1);
  while Pos('{',cStrip) > 0 do delete(cStrip,Pos('{',cStrip),1);
  while Pos('}',cStrip) > 0 do delete(cStrip,Pos('}',cStrip),1);

  Result := cStrip;
end;


{ Function to backup tag file }
function BackupTags(cTagFileName: string): Boolean;
begin
  Result := False;
  if FileExists(cBakFile) then
    begin
      if not DeleteFile(cBakFile) then
        begin
          MessageDlg('Error Erasing Old Backup File!',mtError,[mbOK],0);
          Main_Frm.Status.Panels[2].Text := 'Idle';
          Screen.Cursor := crDefault;
          Exit;
        end;
    end;
  if not RenameFile(cTagFileName,cBakFile) then
    begin
      MessageDlg('Error Creating Backup File!',mtError,[mbOK],0);
      Main_Frm.Status.Panels[2].Text := 'Idle';
      Screen.Cursor := crDefault;
      Exit;
    end;

  Result := True;
end;



{ Sorts taglines }
procedure SortTags;
var
  liSortList : TStringList;            // Sortable list
  fSortFile  : string;                 // filename for sort
begin
  if (Main_Frm.OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;

  if lParam = False then Sort_Frm.Show;
  Main_Frm.Status.Panels[2].Text := 'Sorting...';
  Screen.Cursor := crHourGlass;

  if lDefFile = True then fSortFile := cDefFile
  else fSortFile := Main_Frm.OpenDialog.Filename;

  if lParam = False then Sort_Frm.Sts_Lbl.Caption := 'Loading...';
  Application.ProcessMessages;
  liSortList := TStringList.Create;
  liSortList.LoadFromFile(fSortFile);

  // Backup tags first
  if BackupTags(fSortFile) = False then
    begin
      Sort_Frm.Close;
      liSortList.Free;
      Exit;
    end;

  if lParam = False then Sort_Frm.Sts_Lbl.Caption := 'Sorting...';
  Application.ProcessMessages;
  liSortList.Sort;

  if lParam = False then Sort_Frm.Sts_Lbl.Caption := 'Writing...';
  Application.ProcessMessages;
  liSortList.SaveToFile(fSortFile);

  // Finish Off
  Main_Frm.Status.Panels[2].Text := 'Idle';
  Screen.Cursor := crDefault;
  liSortList.Free;
  if lParam = False then Sort_Frm.Close;
end;



{ Removes duplicates from tagline file }
procedure DeDupeTags;
var
  fDupeFile : TextFile;                     // Filename of de-dupe file
  fTempFile : TextFile;                     // Temp file to hold de-duped tags
  cFileName : string;                       // Name of de-dupe file
  zBuf      : array[1..20*1024] of Char;    // 20k buffer for speed
  iIsDupe   : integer;                      // Is it a dupe?
  iDupes    : integer;                      // Dupes found so far
  iCount    : integer;                      // counter for loop
  cComp1, cComp2 : string;                  // Strings to compare after stripping
  cOrig1, cOrig2 : string;                  // Original strings to keep
begin
  if (Main_Frm.OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;

  SortTags;         // Sort First
  Main_Frm.Status.Panels[2].Text := 'De-Duping...';

  if lDefFile = True then AssignFile(fDupeFile, cDefFile)
  else AssignFile(fDupeFile,Main_Frm.OpenDialog.FileName);

  System.SetTextBuf(fDupeFile,zBuf);
  iCount := 0;

  // Count the tags
  if lDefFile = True then cFileName := cDefFile
  else cFileName := Main_Frm.OpenDialog.FileName;
  CountTags(cFileName);

  // Now we do the de-dupe
  AssignFile(fTempFile,AddName2Path(cCurrDir,'TFactory.$$$'));
  Reset(fDupeFile);
  if lParam = False then DeDupe_Frm.Show;
  if lParam = False then DeDupe_Frm.Gauge.MaxValue := iTotalTags;
  Application.ProcessMessages;
  ReWrite(fTempFile);
  Screen.Cursor := crHourGlass;
  iDupes := 0;
  if lParam = False then DeDupe_Frm.HowMany_Lbl.Caption := '0 Dupes Found...';

  while not Eof(fDupeFile) do
    begin
      Inc(iCount);
      if iCount = 1 then Readln(fDupeFile,cOrig1);
      // write out first line
      if iCount = 1 then Writeln(fTempFile,cOrig1);
      cComp1 := StrpPunc(cOrig1);
      Readln(fDupeFile,cOrig2);
      cComp2 := StrpPunc(cOrig2);
      if lParam = False then DeDupe_Frm.Gauge.Progress := DeDupe_Frm.Gauge.Progress + 1;
      Application.ProcessMessages;

      iIsDupe := Pos(UpperCase(cComp2),UpperCase(cComp1));

      // Treat blank lines as dupes
      if (cComp1 = '') and (cComp2 = '') then iIsDupe := 1;

      if iIsDupe = 0 then      // If not a dupe
        begin
          Writeln(fTempFile,cOrig2);
          cOrig1 := cOrig2;
          cComp1 := cComp2;
        end
      else
        begin
          Main_Frm.Scr.Lines.Append(cOrig2);
          Inc(iDupes);
          if lParam = False then
            begin
              DeDupe_Frm.HowMany_Lbl.Caption := IntToStr(iDupes)+' Dupes Found...';
            end;
        end;
    end;

  CloseFile(fDupeFile);
  CloseFile(fTempFile);

  // Now replace the old file with the new one
  System.Erase(fDupeFile);
  System.Rename(fTempFile,cFileName);

  // tidy up and finish off
  if lParam = False then DeDupe_Frm.Gauge.Progress := DeDupe_Frm.Gauge.Progress + 1;
  Screen.Cursor := crDefault;
  if lParam = False then
    begin
      MessageDlg(IntToStr(iDupes)+' Duplicates Found Out Of '+IntToStr(iTotalTags)+
           ' Taglines.',mtInformation,[mbOK],0);
    end;

  if lParam = False then DeDupe_Frm.Hide;
  CountTags(cFileName);
  Main_Frm.Status.Panels[2].Text := 'Idle';
end;



{ Searches for strings and replaces with new ones }
procedure SearchReplace;
var
  fRepFile  : TextFile;              // Handle for tagfile
  fTempFile : TextFile;              // Temp file to hold replaced tags
  zBuf: array[1..20*1024] of Char;   // 20k buffer for speed
  iIsMatch  : integer;               // Have we found a match?
  iCases    : integer;               // How many have we found
  n         : integer;               // Counter for strings in lists
  cTempStr  : string;                // Temporary string
  cFileName : string;                // Name of tagfile
begin
  if (Main_Frm.OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;

  Main_Frm.Status.Panels[2].Text := 'Replacing...';

  if lDefFile = True then
    begin
      AssignFile(fRepFile,cDefFile);
      cFileName := cDefFile;
    end
  else
    begin
      AssignFile(fRepFile,Main_Frm.OpenDialog.FileName);
      cFileName := Main_Frm.OpenDialog.FileName;
    end;

  // Backup tagline file
  CopyFile(cFileName,cBakFile);

  System.SetTextBuf(fRepFile,zBuf);

  // Count the tags
  CountTags(cFileName);

  // Now do the replacing
  if lParam = False then
    begin
      SR_Frm.Show;
      SR_Frm.Found_Lbl.Caption := '0 Cases Identified';
      SR_Frm.Gauge.MaxValue := iTotalTags;
    end;
  Screen.Cursor := crHourGlass;
  Application.ProcessMessages;
  iCases := 0;
  n := 0;

  while n < liSearch.Count do
    begin
      AssignFile(fTempFile,AddName2Path(cCurrDir,'TFactory.$$$'));
      Reset(fRepFile);
      ReWrite(fTempFile);
      while not Eof(fRepFile) do
        begin
          Application.ProcessMessages;
          if lParam = False then
            begin
              SR_Frm.Srch_Lbl.Caption := liSearch.Strings[n];
              SR_Frm.Repl_Lbl.Caption := liReplace.Strings[n];
            end;
          Readln(fRepFile,cTempStr);
          iIsMatch := Pos(liSearch.Strings[n],cTempStr);

          if iIsMatch <> 0 then
            begin
              Delete(cTempStr,iIsMatch,Length(liSearch.Strings[n]));
              Insert(liReplace.Strings[n],cTempStr,iIsMatch);
              Inc(iCases);
              if lParam = False then SR_Frm.Found_Lbl.Caption := IntToStr(iCases)+
                  ' Cases Identified';
              Application.ProcessMessages;
            end;

          Writeln(fTempFile,cTempStr);
          if lParam = False then SR_Frm.Gauge.Progress := SR_Frm.Gauge.Progress + 1;
          Application.ProcessMessages;
        end;

        Inc(n);
        if lParam = False then SR_Frm.Gauge.Progress := 0;
        Application.ProcessMessages;

        CloseFile(fRepFile);
        CloseFile(fTempFile);

        // Now replace the old file with the new one
        System.Erase(fRepFile);
        System.Rename(fTempFile,cFileName);
    end;

  // tidy up and finish off
  Screen.Cursor := crDefault;
  if lParam = False then
    begin
      MessageDlg(IntToStr(iCases)+' Strings Replaced Out Of '+IntToStr(iTotalTags)+
           ' Taglines.',mtInformation,[mbOK],0);
    end;

  if lParam = False then SR_Frm.Hide;
  Main_Frm.Status.Panels[2].Text := 'Idle';
end;


{ Removes tags over a certain length }
procedure RemoveLongOnes;
var
  fLongFile : TextFile;                     // Filename of tagline file
  fTempFile : TextFile;                     // Temp file to hold short tags
  cFileName : string;                       // Name of tag file
  zBuf      : array[1..20*1024] of Char;    // 20k buffer for speed
  iLongOnes : integer;                      // long ones found so far
  cTempStr  : string;                       // Temp string for reading
begin
  if (Main_Frm.OpenDialog.FileName = '') and (lDefFile = False) then
    begin
      MessageDlg('No Tagline File Selected!',mtError,[mbOK],0);
      Exit;
    end;

  if lDefFile = True then AssignFile(fLongFile,cDefFile)
  else AssignFile(fLongFile,Main_Frm.OpenDialog.FileName);

  if lDefFile = True then cFileName := cDefFile
  else cFileName := Main_Frm.OpenDialog.FileName;

  // Sort first
  SortTags;
  Main_Frm.Status.Panels[2].Text := 'Removing Long Tags...';

  System.SetTextBuf(fLongFile,zBuf);

  // Count the tags
  CountTags(cFileName);

  // Now we do the removing
  AssignFile(fTempFile, AddName2Path(cCurrDir,'TFactory.$$$'));
  Reset(fLongFile);
  if lParam = False then
    begin
      Long_Frm.Show;
      Long_Frm.Gauge.MaxValue := iTotalTags;
    end;
  Application.ProcessMessages;
  ReWrite(fTempFile);
  Screen.Cursor := crHourGlass;
  iLongOnes := 0;
  if lParam = False then Long_Frm.HowMany_Lbl.Caption := '0 Long Tags Found...';

  while not Eof(fLongFile) do
    begin
      Readln(fLongFile,cTempStr);
      if lParam = False then Long_Frm.Gauge.Progress := Long_Frm.Gauge.Progress + 1;
      Application.ProcessMessages;

      if Length(cTempStr) <= iMaxLen then      // If not too long write it
        begin
          Writeln(fTempFile,cTempStr);
        end
      else
        begin
          Main_Frm.Scr.Lines.Append(cTempStr);
          Inc(iLongOnes);
          if lParam = False then
            begin
              Long_Frm.HowMany_Lbl.Caption := IntToStr(iLongOnes)+' Long Tags Found...';
            end;
        end;
    end;

  CloseFile(fLongFile);
  CloseFile(fTempFile);

  // Now replace the old file with the new one
  System.Erase(fLongFile);
  System.Rename(fTempFile,cFileName);

  // tidy up and finish off
  Screen.Cursor := crDefault;
  if lParam = False then
    begin
      MessageDlg(IntToStr(iLongOnes)+' Long Taglines Found Out Of '+IntToStr(iTotalTags)+
           ' Taglines.',mtInformation,[mbOK],0);
      Long_Frm.Hide;
    end;

  CountTags(cFileName);
  Main_Frm.Status.Panels[2].Text := 'Idle';
end;


{ Count the total taglines in a file }
function CountTags(TagFile: string): integer;
var
  cTemp     : string;           // Temp string
  CountFile : TextFile;         // File we are counting
begin
  Screen.Cursor := crHourGlass;
  Main_Frm.Status.Panels[1].Text := 'Counting';
  iTotalTags := 0;
  AssignFile(CountFile,TagFile);
  Reset(CountFile);
  while not Eof(CountFile) do
    begin
      Readln(CountFile,cTemp);
      Inc(iTotalTags);
    end;
  CloseFile(CountFile);
  IniFile.WriteString('Totals','TagCount',IntToStr(iTotalTags));
  Main_Frm.Status.Panels[1].Text := IntToStr(iTotalTags);
  Result := iTotalTags;
  Screen.Cursor := crDefault;
end;



end.
